import os, sys, stat
from paramiko import SSHClient, SSHException

host = 'host.example.com'
user = 'username'

def message(message, *extras):
	if verbosity:
		extras = " ".join(extras)
		print message, extras


class sftpClient:
	createdFolders = []
	client = False
	sftp = False

	def __init__(self, basedir=""):
		self.host = host
		self.user = user
		self.basedir = basedir
	
	#this shouldn't be necessary
	#but if we don't close, scripts will hang at their end
	#they still hang if there's an exception after opening a connection
	def close(self):
		if self.client:
			try:
				self.client.close()
			except:
				exctype, excvalue = sys.exc_info()[:2] 
				print "Could not close SSH client:", excvalue

	def ensureConnection(self):
		if not self.sftp:
			try:
				self.openConnection()
				opened = True
			except SSHException, errtext:
				print "Unable to open SSH connection:", errtext
				opened = False
		else:
			opened = True

		return opened

	def openConnection(self):
		#close existing connection if there is one
		if self.client:
			self.client.close()
		self.client = SSHClient()
		self.client.load_system_host_keys()
		self.client.connect(hostname=self.host, username=self.user)
		self.sftp = self.client.open_sftp()
	
	def ensuredir(self, filepath):
		dir = os.path.dirname(filepath)
		return self.mkdir(dir)

	def mkdir(self, path):
		if not path in self.createdFolders:
			success = False
			if self.ensureConnection():
				self.createdFolders.append(path)
				try:
					#self.sftp.mkdir(path)
					mkdir = '/bin/mkdir -p "' + path + '"'
					mkdirIn, mkdirOut, mkdirErr = self.client.exec_command(mkdir)
					for line in mkdirOut:
						print line
					success = True
				except IOError, errtext:
					#there's an empty error if the folder already exists (using sftp.mkdir, anyway)
					#there ought to be a better way to do this
					#really need a .mkdirs() method
					#this runs the risk of ignoring errors when a folder can't be created
					if str(errtext) != "Failure":
						print "'" + str(errtext) + "'"
						excepttype, exceptvalue = sys.exc_info()[:2]
						print "IOError creating directory", path, "-", errtext
						print "\t", exceptvalue
					success = True
				except EOFError, errtext:
					excepttype, exceptvalue = sys.exc_info()[:2]
					print "EOFError creating directory", path, errtext
					print exceptvalue
				except SSHException, errtext:
					print "SSHException creating directory", path, "-", errtext
					#this might mean the connection was closed
					print "\tMarking SFTP as gone" 
					self.sftp = None

			return success
		return True
	

	def put(self, localpath, remotepath):
		success = False
		if self.ensureConnection():
			fileinfo = os.stat(localpath)
			try:
				self.sftp.put(localpath, remotepath)
				success = True
			except IOError, errtext:
				print "IOError uploading", localpath, "to", remotepath, "-", errtext
			except EOFError, errtext:
				print "EOFError uploading", localpath, "to", remotepath, "-", errtext
			except SSHException, errtext:
				print "SSHException uploading", localpath, "to", remotepath, "-", errtext
				#this might mean the connection was closed
				print "\tMarking SFTP as gone" 
				self.sftp = None
		
			if success:
				try:
					self.sftp.utime(remotepath, (fileinfo.st_atime, fileinfo.st_mtime))
				except EOFError, errtext:
					#note the error, but this is hardly a deal-killer
					print "EOFError setting timestamp for", remotepath, errtext
				except IOError, errtext:
					print "IOError setting timestamp for", remotepath, errtext

		return success

	def fullpath(self, path):
		return self.basedir + path

	def remove(self, path):
		success = False
		if self.ensureConnection():
			path = self.fullpath(path)
			try:
				self.sftp.remove(path)
				success = True
			except IOError, errtext:
				excepttype, exceptvalue = sys.exc_info()[:2]
				print "\tIOError removing", path, errtext
				if exceptvalue.errno == 2:
					#the file doesn't exist, so we don't need to delete it
					print "\tBut that's cool."
					success = True
				else:
					print "\t", exceptvalue.errno, exceptvalue.strerror

		return success
	
	def rmdir(self, path):
		success = True
		print "\tOnline removal of folders not yet implemented:", path
		return success

	def uploadString(self, pageText, stagefile, remotefile, mtime=None):
		if self.ensureConnection():
			destination = self.fullpath(remotefile)
			if self.ensuredir(destination):
				message("\tUploading string to", remotefile)
				try:
					uploader = self.sftp.file(destination, mode='w')
					uploader.write(pageText)
					uploader.close()
				except IOError, errtext:
					print
					print "IOError writing file", destination, "-", errtext
					return False
				except EOFError, errtext:
					print
					print "EOF error writing file", destination, "-", errtext
					return False
				except SSHException, errtext:
					print
					print "SSHException writing file", destination, "-", errtext
					return False
	
				
				if mtime:
					self.sftp.utime(destination, (mtime, mtime))
					
				message("\tUploaded to", destination)
				
				#if we have made it this far, save the cache file so that we can check it in the future
				localDir = os.path.dirname(stagefile)
				print "Saving", stagefile
				if not os.path.exists(localDir):
					print "\tCreating folder", localDir
					os.makedirs(localDir)
	
	 			writeFile = open(stagefile, 'w')
	 			writeFile.write(pageText)
	 			writeFile.close()
	 			
				return True

		return False
